(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('leaflet'), require('react'), require('react-dom')) :
	typeof define === 'function' && define.amd ? define(['exports', 'leaflet', 'react', 'react-dom'], factory) :
	(global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.ReactLeaflet = {}, global.L, global.React));
}(this, (function (exports, L$1, React) { 'use strict';

	(function () {

		/**
		 * GoogleMutant by Iván Sánchez Ortega <ivan@sanchezortega.es> https://ivan.sanchezortega.es
		 * Source and issue tracking: https://gitlab.com/IvanSanchez/Leaflet.GridLayer.GoogleMutant/
		 *
		 * Based on techniques from https://github.com/shramov/leaflet-plugins
		 * and https://avinmathew.com/leaflet-and-google-maps/ , but relying on MutationObserver.
		 *
		 * "THE BEER-WARE LICENSE":
		 * <ivan@sanchezortega.es> wrote this file. As long as you retain this notice you
		 * can do whatever you want with this stuff. If we meet some day, and you think
		 * this stuff is worth it, you can buy me a beer in return.
		 *
		 * Uses MIT-licensed code from https://github.com/rsms/js-lru/
		 */

		// This implementation of LRUMap is a copy of https://github.com/rsms/js-lru/ ,
		// trivially adapted for ES6 exports.

		/*
		The MIT License

		Copyright (c) 2010-2020 Rasmus Andersson <https://rsms.me/>

		Permission is hereby granted, free of charge, to any person obtaining a copy
		of this software and associated documentation files (the "Software"), to deal
		in the Software without restriction, including without limitation the rights
		to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
		copies of the Software, and to permit persons to whom the Software is
		furnished to do so, subject to the following conditions:

		The above copyright notice and this permission notice shall be included in
		all copies or substantial portions of the Software.

		THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
		IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
		FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
		AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
		LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
		OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
		THE SOFTWARE.
		*/

		/**
		 * A doubly linked list-based Least Recently Used (LRU) cache. Will keep most
		 * recently used items while discarding least recently used items when its limit
		 * is reached.
		 *
		 * Licensed under MIT. Copyright (c) 2010 Rasmus Andersson <http://hunch.se/>
		 * See README.md for details.
		 *
		 * Illustration of the design:
		 *
		 *       entry             entry             entry             entry
		 *       ______            ______            ______            ______
		 *      | head |.newer => |      |.newer => |      |.newer => | tail |
		 *      |  A   |          |  B   |          |  C   |          |  D   |
		 *      |______| <= older.|______| <= older.|______| <= older.|______|
		 *
		 *  removed  <--  <--  <--  <--  <--  <--  <--  <--  <--  <--  <--  added
		 */

		var NEWER = Symbol("newer");
		var OLDER = Symbol("older");

		var LRUMap = function LRUMap(limit, entries) {
			if (typeof limit !== "number") {
				// called as (entries)
				entries = limit;
				limit = 0;
			}

			this.size = 0;
			this.limit = limit;
			this.oldest = this.newest = undefined;
			this._keymap = new Map();

			if (entries) {
				this.assign(entries);
				if (limit < 1) {
					this.limit = this.size;
				}
			}
		};

		LRUMap.prototype._markEntryAsUsed = function _markEntryAsUsed(entry) {
			if (entry === this.newest) {
				// Already the most recenlty used entry, so no need to update the list
				return;
			}
			// HEAD--------------TAIL
			//   <.older   .newer>
			//  <--- add direction --
			//   A  B  C  <D>  E
			if (entry[NEWER]) {
				if (entry === this.oldest) {
					this.oldest = entry[NEWER];
				}
				entry[NEWER][OLDER] = entry[OLDER]; // C <-- E.
			}
			if (entry[OLDER]) {
				entry[OLDER][NEWER] = entry[NEWER]; // C. --> E
			}
			entry[NEWER] = undefined; // D --x
			entry[OLDER] = this.newest; // D. --> E
			if (this.newest) {
				this.newest[NEWER] = entry; // E. <-- D
			}
			this.newest = entry;
		};

		LRUMap.prototype.assign = function assign(entries) {
			var entry,
				limit = this.limit || Number.MAX_VALUE;
			this._keymap.clear();
			var it = entries[Symbol.iterator]();
			for (var itv = it.next(); !itv.done; itv = it.next()) {
				var e = new Entry(itv.value[0], itv.value[1]);
				this._keymap.set(e.key, e);
				if (!entry) {
					this.oldest = e;
				} else {
					entry[NEWER] = e;
					e[OLDER] = entry;
				}
				entry = e;
				if (limit-- == 0) {
					throw new Error("overflow");
				}
			}
			this.newest = entry;
			this.size = this._keymap.size;
		};

		LRUMap.prototype.get = function get(key) {
			// First, find our cache entry
			var entry = this._keymap.get(key);
			if (!entry) {
				return;
			} // Not cached. Sorry.
			// As <key> was found in the cache, register it as being requested recently
			this._markEntryAsUsed(entry);
			return entry.value;
		};

		LRUMap.prototype.set = function set(key, value) {
			var entry = this._keymap.get(key);

			if (entry) {
				// update existing
				entry.value = value;
				this._markEntryAsUsed(entry);
				return this;
			}

			// new entry
			this._keymap.set(key, (entry = new Entry(key, value)));

			if (this.newest) {
				// link previous tail to the new tail (entry)
				this.newest[NEWER] = entry;
				entry[OLDER] = this.newest;
			} else {
				// we're first in -- yay
				this.oldest = entry;
			}

			// add new entry to the end of the linked list -- it's now the freshest entry.
			this.newest = entry;
			++this.size;
			if (this.size > this.limit) {
				// we hit the limit -- remove the head
				this.shift();
			}

			return this;
		};

		LRUMap.prototype.shift = function shift() {
			// todo: handle special case when limit == 1
			var entry = this.oldest;
			if (entry) {
				if (this.oldest[NEWER]) {
					// advance the list
					this.oldest = this.oldest[NEWER];
					this.oldest[OLDER] = undefined;
				} else {
					// the cache is exhausted
					this.oldest = undefined;
					this.newest = undefined;
				}
				// Remove last strong reference to <entry> and remove links from the purged
				// entry being returned:
				entry[NEWER] = entry[OLDER] = undefined;
				this._keymap.delete(entry.key);
				--this.size;
				return [entry.key, entry.value];
			}
		};

		// -------------------------------------------------------------------------------------
		// Following code (until end of class definition) is optional and can be removed without
		// breaking the core functionality.

		LRUMap.prototype.find = function find(key) {
			var e = this._keymap.get(key);
			return e ? e.value : undefined;
		};

		LRUMap.prototype.has = function has(key) {
			return this._keymap.has(key);
		};

		LRUMap.prototype.delete = function delete$1(key) {
			var entry = this._keymap.get(key);
			if (!entry) {
				return;
			}
			this._keymap.delete(entry.key);
			if (entry[NEWER] && entry[OLDER]) {
				// relink the older entry with the newer entry
				entry[OLDER][NEWER] = entry[NEWER];
				entry[NEWER][OLDER] = entry[OLDER];
			} else if (entry[NEWER]) {
				// remove the link to us
				entry[NEWER][OLDER] = undefined;
				// link the newer entry to head
				this.oldest = entry[NEWER];
			} else if (entry[OLDER]) {
				// remove the link to us
				entry[OLDER][NEWER] = undefined;
				// link the newer entry to head
				this.newest = entry[OLDER];
			} else {
				// if(entry[OLDER] === undefined && entry.newer === undefined) {
				this.oldest = this.newest = undefined;
			}

			this.size--;
			return entry.value;
		};

		LRUMap.prototype.clear = function clear() {
			// Not clearing links should be safe, as we don't expose live links to user
			this.oldest = this.newest = undefined;
			this.size = 0;
			this._keymap.clear();
		};

		LRUMap.prototype.keys = function keys() {
			return new KeyIterator(this.oldest);
		};

		LRUMap.prototype.values = function values() {
			return new ValueIterator(this.oldest);
		};

		LRUMap.prototype.entries = function entries() {
			return this;
		};

		LRUMap.prototype[Symbol.iterator] = function () {
			return new EntryIterator(this.oldest);
		};

		LRUMap.prototype.forEach = function forEach(fun, thisObj) {
			if (typeof thisObj !== "object") {
				thisObj = this;
			}
			var entry = this.oldest;
			while (entry) {
				fun.call(thisObj, entry.value, entry.key, this);
				entry = entry[NEWER];
			}
		};

		/** Returns a JSON (array) representation */
		LRUMap.prototype.toJSON = function toJSON() {
			var s = new Array(this.size),
				i = 0,
				entry = this.oldest;
			while (entry) {
				s[i++] = { key: entry.key, value: entry.value };
				entry = entry[NEWER];
			}
			return s;
		};

		/** Returns a String representation */
		LRUMap.prototype.toString = function toString() {
			var s = "",
				entry = this.oldest;
			while (entry) {
				s += String(entry.key) + ":" + entry.value;
				entry = entry[NEWER];
				if (entry) {
					s += " < ";
				}
			}
			return s;
		};

		function Entry(key, value) {
			this.key = key;
			this.value = value;
			this[NEWER] = undefined;
			this[OLDER] = undefined;
		}

		function EntryIterator(oldestEntry) {
			this.entry = oldestEntry;
		}
		EntryIterator.prototype[Symbol.iterator] = function () {
			return this;
		};
		EntryIterator.prototype.next = function () {
			var ent = this.entry;
			if (ent) {
				this.entry = ent[NEWER];
				return { done: false, value: [ent.key, ent.value] };
			} else {
				return { done: true, value: undefined };
			}
		};

		function KeyIterator(oldestEntry) {
			this.entry = oldestEntry;
		}
		KeyIterator.prototype[Symbol.iterator] = function () {
			return this;
		};
		KeyIterator.prototype.next = function () {
			var ent = this.entry;
			if (ent) {
				this.entry = ent[NEWER];
				return { done: false, value: ent.key };
			} else {
				return { done: true, value: undefined };
			}
		};

		function ValueIterator(oldestEntry) {
			this.entry = oldestEntry;
		}
		ValueIterator.prototype[Symbol.iterator] = function () {
			return this;
		};
		ValueIterator.prototype.next = function () {
			var ent = this.entry;
			if (ent) {
				this.entry = ent[NEWER];
				return { done: false, value: ent.value };
			} else {
				return { done: true, value: undefined };
			}
		};

		// GoogleMutant by Iván Sánchez Ortega <ivan@sanchezortega.es>

		var GAPIPromise = new Promise(function (resolve, reject) {
			var checkCounter = 0,
				intervalId = null;

			intervalId = setInterval(function () {
				if (checkCounter >= 20) {
					clearInterval(intervalId);
					return reject(new Error("window.google not found after 10 seconds"));
				}
				if (!!window.google && !!window.google.maps && !!window.google.maps.Map) {
					clearInterval(intervalId);
					return resolve(window.google);
				}
				++checkCounter;
			}, 500);
		});

		// 🍂class GridLayer.GoogleMutant
		// 🍂extends GridLayer
		L.GridLayer.GoogleMutant = L.GridLayer.extend({
			options: {
				minZoom: 0,
				maxZoom: 21, // can be 23, but ugly if more than maxNativeZoom
				tileSize: 256,
				subdomains: "abc",
				errorTileUrl: "",
				attribution: "", // The mutant container will add its own attribution anyways.
				opacity: 1,
				continuousWorld: false,
				noWrap: false,
				// 🍂option type: String = 'roadmap'
				// Google's map type. Valid values are 'roadmap', 'satellite' or 'terrain'. 'hybrid' is not really supported.
				type: "roadmap",
				maxNativeZoom: 21,
			},

			initialize: function (options) {
				L.GridLayer.prototype.initialize.call(this, options);

				this._isMounted = true;

				this.once("spawned", function () {
					if (this._subLayers) {
						//restore previously added google layers
						for (var layerName in this._subLayers) {
							this._subLayers[layerName].setMap(this._mutant);
						}
					}
				});

				// Couple data structures indexed by tile key
				this._tileCallbacks = {}; // Callbacks for promises for tiles that are expected
				this._lru = new LRUMap(100); // Tile LRU cache

				this._imagesPerTile = this.options.type === "hybrid" ? 2 : 1;

				this._boundOnMutatedImage = this._onMutatedImage.bind(this);
			},

			onAdd: function (map) {
				var this$1 = this;

				L.GridLayer.prototype.onAdd.call(this, map);
				this._initMutantContainer();

				// Attribution and logo nodes are not mutated a second time if the
				// mutant is removed and re-added to the map, hence they are
				// not cleaned up on layer removal, so they can be added here.
				if (this._logoContainer) {
					map._controlCorners.bottomleft.appendChild(this._logoContainer);
				}
				if (this._attributionContainer) {
					map._controlCorners.bottomright.appendChild(this._attributionContainer);
				}

				GAPIPromise.then(function () {
					if (!this$1._isMounted) {
						return;
					}
					this$1._ready = true;

					this$1._initMutant();

					map = this$1._map;
					if (!map) {
						return;
					}
					var moveevent = this$1.options.updateWhenIdle ? "moveend" : "move";
					map.on(moveevent, this$1._update, this$1);
					this$1.once("remove", function () {
						this._map.off(moveevent, this._update, this);
					});
					//handle layer being added to a map for which there are no Google tiles at the given zoom
					google.maps.event.addListenerOnce(this$1._mutant, "idle", function () {
						if (!this$1._map) {
							return;
						}
						this$1._checkZoomLevels();
						this$1._mutantIsReady = true;
					});

					this$1._update();
				});
			},

			onRemove: function (map) {
				L.GridLayer.prototype.onRemove.call(this, map);
				this._observer.disconnect();
				map._container.removeChild(this._mutantContainer);
				if (this._logoContainer) {
					L.DomUtil.remove(this._logoContainer);
				}
				if (this._attributionContainer) {
					L.DomUtil.remove(this._attributionContainer);
				}

				google.maps.event.clearListeners(map, "idle");
				if (this._mutant) {
					google.maps.event.clearListeners(this._mutant, "idle");
				}
				map.off("move moveend", this._update, this);

				this._isMounted = false;
			},

			// 🍂method addGoogleLayer(name: String, options?: Object): this
			// Adds layer with the given name and options to the google Map instance.
			// `name`: one of the google maps API layers, with it's constructor available in `google.maps` object.
			// currently following values supported: 'TrafficLayer', 'TransitLayer', 'BicyclingLayer'.
			// `options`: see https://developers.google.com/maps/documentation/javascript/reference/map
			addGoogleLayer: function (googleLayerName, options) {
				var this$1 = this;

				if (!this._subLayers) {
					this._subLayers = {};
				}
				GAPIPromise.then(function () {
					var Constructor = google.maps[googleLayerName];
					var googleLayer = new Constructor(options);
					if (this$1._mutant) {
						googleLayer.setMap(this$1._mutant);
					} // otherwise it will be added on 'spawned'
					this$1._subLayers[googleLayerName] = googleLayer;
				});
				return this;
			},

			// 🍂method removeGoogleLayer(name: String): this
			// Removes layer with the given name from the google Map instance.
			removeGoogleLayer: function (googleLayerName) {
				var this$1 = this;

				GAPIPromise.then(function () {
					var googleLayer = this$1._subLayers && this$1._subLayers[googleLayerName];
					if (googleLayer) {
						googleLayer.setMap(null);
						delete this$1._subLayers[googleLayerName];
					}
				});
				return this;
			},

			_initMutantContainer: function () {
				if (!this._mutantContainer) {
					this._mutantContainer = L.DomUtil.create(
						"div",
						"leaflet-google-mutant leaflet-top leaflet-left"
					);
					this._mutantContainer.id =
						"_MutantContainer_" + L.Util.stamp(this._mutantContainer);
					this._mutantContainer.style.zIndex = 800; //leaflet map pane at 400, controls at 1000
					this._mutantContainer.style.pointerEvents = "none";

					L.DomEvent.off(this._mutantContainer);
				}
				this._map.getContainer().appendChild(this._mutantContainer);

				this.setOpacity(this.options.opacity);
				var style = this._mutantContainer.style;
				if (this.options.zoomSnap < 1) {
					// Fractional zoom needs a bigger mutant container in order to load more (smaller) tiles
					style.width = "150%";
					style.height = "150%";
				} else {
					style.width = "100%";
					style.height = "100%";
				}
				style.zIndex = -1;

				this._attachObserver(this._mutantContainer);
			},

			_initMutant: function () {
				var this$1 = this;

				if (this._mutant) {
					// reuse old _mutant, just make sure it has the correct size
					return;
				}

				var map = new google.maps.Map(this._mutantContainer, {
					center: { lat: 0, lng: 0 },
					zoom: 0,
					tilt: 0,
					mapTypeId: this.options.type,
					disableDefaultUI: true,
					keyboardShortcuts: false,
					draggable: false,
					disableDoubleClickZoom: true,
					scrollwheel: false,
					streetViewControl: false,
					styles: this.options.styles || {},
					backgroundColor: "transparent",
				});

				this._mutant = map;

				google.maps.event.addListenerOnce(map, "idle", function () {
					var nodes = this$1._mutantContainer.querySelectorAll("a");
					for (var i = 0; i < nodes.length; ++i) {
						nodes[i].style.pointerEvents = "auto";
					}
				});

				// 🍂event spawned
				// Fired when the mutant has been created.
				this.fire("spawned", { mapObject: map });
			},

			_attachObserver: function _attachObserver(node) {
				if (!this._observer) {
					this._observer = new MutationObserver(this._onMutations.bind(this));
				}

				// pass in the target node, as well as the observer options
				this._observer.observe(node, { childList: true, subtree: true });

				// if we are reusing an old _mutantContainer, we must manually detect
				// all existing tiles in it
				Array.prototype.forEach.call(node.querySelectorAll("img"), this._boundOnMutatedImage);
			},

			_onMutations: function _onMutations(mutations) {
				for (var i = 0; i < mutations.length; ++i) {
					var mutation = mutations[i];
					for (var j = 0; j < mutation.addedNodes.length; ++j) {
						var node = mutation.addedNodes[j];

						if (node instanceof HTMLImageElement) {
							this._onMutatedImage(node);
						} else if (node instanceof HTMLElement) {
							Array.prototype.forEach.call(
								node.querySelectorAll("img"),
								this._boundOnMutatedImage
							);

							// Check for, and remove, the "Google Maps can't load correctly" div.
							// You *are* loading correctly, you dumbwit.
							if (node.style.backgroundColor === "white") {
								L.DomUtil.remove(node);
							}

							// Check for, and remove, the "For development purposes only" divs on the aerial/hybrid tiles.
							if (node.textContent.indexOf("For development purposes only") === 0) {
								L.DomUtil.remove(node);
							}

							// Check for, and remove, the "Sorry, we have no imagery here"
							// empty <div>s. The [style*="text-align: center"] selector
							// avoids matching the attribution notice.
							// This empty div doesn't have a reference to the tile
							// coordinates, so it's not possible to mark the tile as
							// failed.
							Array.prototype.forEach.call(
								node.querySelectorAll(
									'div[draggable=false][style*="text-align: center"]'
								),
								L.DomUtil.remove
							);

							// Move Google attributions to leaflet's bottom-right control container
							if (
								node.querySelectorAll(".gmnoprint").length > 0 ||
								node.querySelectorAll(
									'a[title="Click to see this area on Google Maps"]'
								).length > 0
							) {
								var ctr = (this._attributionContainer = L.DomUtil.create(
									"div",
									"leaflet-control leaflet-control-attribution"
								));
								L.DomEvent.disableClickPropagation(ctr);
								ctr.style.height = "14px";
								this._map._controlCorners.bottomright.appendChild(ctr);
								ctr.appendChild(node);
							}

							// Move Google logo to leaflet's bottom-left control container
							if (node.style.zIndex == 1000000) {
								this._map._controlCorners.bottomleft.appendChild(node);
								this._logoContainer = node;
							}
						}
					}
				}
			},

			// Only images which 'src' attrib match this will be considered for moving around.
			// Looks like some kind of string-based protobuf, maybe??
			// Only the roads (and terrain, and vector-based stuff) match this pattern
			_roadRegexp: /!1i(\d+)!2i(\d+)!3i(\d+)!/,

			// On the other hand, raster imagery matches this other pattern
			_satRegexp: /x=(\d+)&y=(\d+)&z=(\d+)/,

			// On small viewports, when zooming in/out, a static image is requested
			// This will not be moved around, just removed from the DOM.
			_staticRegExp: /StaticMapService\.GetMapImage/,

			_onMutatedImage: function _onMutatedImage(imgNode) {
				var coords;
				var match = imgNode.src.match(this._roadRegexp);
				var sublayer = 0;

				if (match) {
					coords = {
						z: match[1],
						x: match[2],
						y: match[3],
					};
					if (this._imagesPerTile > 1) {
						imgNode.style.zIndex = 1;
						sublayer = 1;
					}
				} else {
					match = imgNode.src.match(this._satRegexp);
					if (match) {
						coords = {
							x: match[1],
							y: match[2],
							z: match[3],
						};
					}
					// imgNode.style.zIndex = 0;
					sublayer = 0;
				}

				if (coords) {
					var tileKey = this._tileCoordsToKey(coords);
					imgNode.style.position = "absolute";
					imgNode.style.visibility = "hidden";

					var key = tileKey + "/" + sublayer;
					// Cache img so it can also be used in subsequent tile requests
					this._lru.set(key, imgNode);

					if (key in this._tileCallbacks && this._tileCallbacks[key]) {
						// Use the tile for *all* pending callbacks. They'll be cloned anyway.
						this._tileCallbacks[key].forEach(function (callback) {
							return callback(imgNode);
						});
						delete this._tileCallbacks[key];
					}
				} else if (imgNode.src.match(this._staticRegExp)) {
					imgNode.style.visibility = "hidden";
				}
			},

			createTile: function (coords, done) {
				var key = this._tileCoordsToKey(coords),
					tileContainer = L.DomUtil.create("div");

				tileContainer.style.textAlign = "left";
				tileContainer.dataset.pending = this._imagesPerTile;
				done = done.bind(this, null, tileContainer);

				for (var i = 0; i < this._imagesPerTile; ++i) {
					var key2 = key + "/" + i,
						imgNode = this._lru.get(key2);
					if (imgNode) {
						tileContainer.appendChild(this._clone(imgNode));
						--tileContainer.dataset.pending;
					} else {
						this._tileCallbacks[key2] = this._tileCallbacks[key2] || [];
						this._tileCallbacks[key2].push(
							function (c /*, k2*/) {
								return function (imgNode) {
									c.appendChild(this._clone(imgNode));
									--c.dataset.pending;
									if (!parseInt(c.dataset.pending)) {
										done();
									}
								}.bind(this);
							}.bind(this)(tileContainer /*, key2*/)
						);
					}
				}

				if (!parseInt(tileContainer.dataset.pending)) {
					L.Util.requestAnimFrame(done);
				}
				return tileContainer;
			},

			_clone: function (imgNode) {
				var clonedImgNode = imgNode.cloneNode(true);
				clonedImgNode.style.visibility = "visible";
				return clonedImgNode;
			},

			_checkZoomLevels: function () {
				//setting the zoom level on the Google map may result in a different zoom level than the one requested
				//(it won't go beyond the level for which they have data).
				var zoomLevel = this._map.getZoom(),
					gMapZoomLevel = this._mutant.getZoom();

				if (!zoomLevel || !gMapZoomLevel) {
					return;
				}

				if (
					gMapZoomLevel !== zoomLevel || //zoom levels are out of sync, Google doesn't have data
					gMapZoomLevel > this.options.maxNativeZoom
				) {
					//at current location, Google does have data (contrary to maxNativeZoom)
					//Update maxNativeZoom
					this._setMaxNativeZoom(gMapZoomLevel);
				}
			},

			_setMaxNativeZoom: function (zoomLevel) {
				if (zoomLevel !== this.options.maxNativeZoom) {
					this.options.maxNativeZoom = zoomLevel;
					this._resetView();
				}
			},

			_update: function () {
				// zoom level check needs to happen before super's implementation (tile addition/creation)
				// otherwise tiles may be missed if maxNativeZoom is not yet correctly determined
				if (this._mutant) {
					var center = this._map.getCenter(),
						_center = new google.maps.LatLng(center.lat, center.lng),
						zoom = Math.round(this._map.getZoom()),
						mutantZoom = this._mutant.getZoom();

					this._mutant.setCenter(_center);

					//ignore fractional zoom levels
					if (zoom !== mutantZoom) {
						this._mutant.setZoom(zoom);

						if (this._mutantIsReady) {
							this._checkZoomLevels();
						}
						//else zoom level check will be done later by 'idle' handler
					}
				}

				L.GridLayer.prototype._update.call(this);
			},

			_resize: function () {
				var factor = this.options.zoomSnap < 1 ? 1.5 : 1;
				var size = this._map.getSize().multiplyBy(factor);
				if (
					this._mutantContainer.style.width === size.x &&
					this._mutantContainer.style.height === size.y
				) {
					return;
				}
				this.setElementSize(this._mutantContainer, size);
				if (!this._mutant) {
					return;
				}
				google.maps.event.trigger(this._mutant, "resize");
			},

			_handleZoomAnim: function () {
				if (!this._mutant) {
					return;
				}

				var center = this._map.getCenter(),
					_center = new google.maps.LatLng(center.lat, center.lng);

				this._mutant.setCenter(_center);
				this._mutant.setZoom(Math.round(this._map.getZoom()));
			},
		});

		// 🍂factory gridLayer.googleMutant(options)
		// Returns a new `GridLayer.GoogleMutant` given its options
		L.gridLayer.googleMutant = function (options) {
			return new L.GridLayer.GoogleMutant(options);
		};
	})();

	function useAttribution(map, attribution) {
	  const attributionRef = React.useRef(attribution);
	  React.useEffect(function updateAttribution() {
	    if (attribution !== attributionRef.current && map.attributionControl != null) {
	      if (attributionRef.current != null) {
	        map.attributionControl.removeAttribution(attributionRef.current);
	      }

	      if (attribution != null) {
	        map.attributionControl.addAttribution(attribution);
	      }
	    }

	    attributionRef.current = attribution;
	  }, [map, attribution]);
	}

	const LeafletContext = /*#__PURE__*/React.createContext(null);
	LeafletContext.Provider;
	function useLeafletContext() {
	  const context = React.useContext(LeafletContext);

	  if (context == null) {
	    throw new Error('No context provided: useLeafletContext() can only be used in a descendant of <MapContainer>');
	  }

	  return context;
	}

	function createLeafComponent(useElement) {
	  function LeafComponent(props, ref) {
	    const {
	      instance
	    } = useElement(props).current;
	    React.useImperativeHandle(ref, () => instance);
	    return null;
	  }

	  return /*#__PURE__*/React.forwardRef(LeafComponent);
	}

	function useEventHandlers(element, eventHandlers) {
	  const eventHandlersRef = React.useRef();
	  React.useEffect(function addEventHandlers() {
	    if (eventHandlers != null) {
	      element.instance.on(eventHandlers);
	    }

	    eventHandlersRef.current = eventHandlers;
	    return function removeEventHandlers() {
	      if (eventHandlersRef.current != null) {
	        element.instance.off(eventHandlersRef.current);
	      }

	      eventHandlersRef.current = null;
	    };
	  }, [element, eventHandlers]);
	}

	function withPane(props, context) {
	  var _props$pane;

	  const pane = (_props$pane = props.pane) != null ? _props$pane : context.pane;
	  return pane ? { ...props,
	    pane
	  } : props;
	}

	function createElementHook(createElement, updateElement) {
	  if (updateElement == null) {
	    return function useImmutableLeafletElement(props, context) {
	      return React.useRef(createElement(props, context));
	    };
	  }

	  return function useMutableLeafletElement(props, context) {
	    const elementRef = React.useRef(createElement(props, context));
	    const propsRef = React.useRef(props);
	    const {
	      instance
	    } = elementRef.current;
	    React.useEffect(function updateElementProps() {
	      if (propsRef.current !== props) {
	        updateElement(instance, props, propsRef.current);
	        propsRef.current = props;
	      }
	    }, [instance, props, context]);
	    return elementRef;
	  };
	}

	function useLayerLifecycle(element, context) {
	  React.useEffect(function addLayer() {
	    var _context$layerContain;

	    const container = (_context$layerContain = context.layerContainer) != null ? _context$layerContain : context.map;
	    container.addLayer(element.instance);
	    return function removeLayer() {
	      container.removeLayer(element.instance);
	    };
	  }, [context, element]);
	}
	function createLayerHook(useElement) {
	  return function useLayer(props) {
	    const context = useLeafletContext();
	    const elementRef = useElement(withPane(props, context), context);
	    useAttribution(context.map, props.attribution);
	    useEventHandlers(elementRef.current, props.eventHandlers);
	    useLayerLifecycle(elementRef.current, context);
	    return elementRef;
	  };
	}

	function createTileLayerComponent(createElement, updateElement) {
	  const useElement = createElementHook(createElement, updateElement);
	  const useLayer = createLayerHook(useElement);
	  return createLeafComponent(useLayer);
	}

	function updateGridLayer(layer, props, prevProps) {
	  const {
	    opacity,
	    zIndex
	  } = props;

	  if (opacity != null && opacity !== prevProps.opacity) {
	    layer.setOpacity(opacity);
	  }

	  if (zIndex != null && zIndex !== prevProps.zIndex) {
	    layer.setZIndex(zIndex);
	  }
	}

	/**
	 * Uses the GoogleLayer Leaflet plugin to create a googlemaps layer
	 * Requires the Google Maps JS API to be loaded as a script tag.
	 *
	 * All options except `opacity` and `zIndex` are immutable once the layer has loaded with the exception
	 *
	 * Docs for L.GridLayer.GoogleMutant https://gitlab.com/IvanSanchez/Leaflet.GridLayer.GoogleMutant
	 */
	const GoogleLayer = createTileLayerComponent((props, context) => {
	  const instance = L$1.gridLayer.googleMutant({ ...props
	  });
	  return {
	    instance,
	    context
	  };
	}, updateGridLayer);

	// do not edit .js files directly - edit src/index.jst



	var fastDeepEqual = function equal(a, b) {
	  if (a === b) return true;

	  if (a && b && typeof a == 'object' && typeof b == 'object') {
	    if (a.constructor !== b.constructor) return false;

	    var length, i, keys;
	    if (Array.isArray(a)) {
	      length = a.length;
	      if (length != b.length) return false;
	      for (i = length; i-- !== 0;)
	        if (!equal(a[i], b[i])) return false;
	      return true;
	    }



	    if (a.constructor === RegExp) return a.source === b.source && a.flags === b.flags;
	    if (a.valueOf !== Object.prototype.valueOf) return a.valueOf() === b.valueOf();
	    if (a.toString !== Object.prototype.toString) return a.toString() === b.toString();

	    keys = Object.keys(a);
	    length = keys.length;
	    if (length !== Object.keys(b).length) return false;

	    for (i = length; i-- !== 0;)
	      if (!Object.prototype.hasOwnProperty.call(b, keys[i])) return false;

	    for (i = length; i-- !== 0;) {
	      var key = keys[i];

	      if (!equal(a[key], b[key])) return false;
	    }

	    return true;
	  }

	  // true if both NaN, false otherwise
	  return a!==a && b!==b;
	};

	/**
	 * Copyright 2019 Google LLC. All Rights Reserved.
	 *
	 * Licensed under the Apache License, Version 2.0 (the "License");
	 * you may not use this file except in compliance with the License.
	 * You may obtain a copy of the License at.
	 *
	 *      Http://www.apache.org/licenses/LICENSE-2.0.
	 *
	 * Unless required by applicable law or agreed to in writing, software
	 * distributed under the License is distributed on an "AS IS" BASIS,
	 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	 * See the License for the specific language governing permissions and
	 * limitations under the License.
	 */
	const DEFAULT_ID = "__googleMapsScriptId";
	/**
	 * [[Loader]] makes it easier to add Google Maps JavaScript API to your application
	 * dynamically using
	 * [Promises](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Promise).
	 * It works by dynamically creating and appending a script node to the the
	 * document head and wrapping the callback function so as to return a promise.
	 *
	 * ```
	 * const loader = new Loader({
	 *   apiKey: "",
	 *   version: "weekly",
	 *   libraries: ["places"]
	 * });
	 *
	 * loader.load().then(() => {
	 *   const map = new google.maps.Map(...)
	 * })
	 * ```
	 */
	class Loader {
	    /**
	     * Creates an instance of Loader using [[LoaderOptions]]. No defaults are set
	     * using this library, instead the defaults are set by the Google Maps
	     * JavaScript API server.
	     *
	     * ```
	     * const loader = Loader({apiKey, version: 'weekly', libraries: ['places']});
	     * ```
	     */
	    constructor({ apiKey, channel, client, id = DEFAULT_ID, libraries = [], language, region, version, mapIds, nonce, retries = 3, url = "https://maps.googleapis.com/maps/api/js", }) {
	        this.CALLBACK = "__googleMapsCallback";
	        this.callbacks = [];
	        this.done = false;
	        this.loading = false;
	        this.errors = [];
	        this.version = version;
	        this.apiKey = apiKey;
	        this.channel = channel;
	        this.client = client;
	        this.id = id || DEFAULT_ID; // Do not allow empty string
	        this.libraries = libraries;
	        this.language = language;
	        this.region = region;
	        this.mapIds = mapIds;
	        this.nonce = nonce;
	        this.retries = retries;
	        this.url = url;
	        if (Loader.instance) {
	            if (!fastDeepEqual(this.options, Loader.instance.options)) {
	                throw new Error(`Loader must not be called again with different options. ${JSON.stringify(this.options)} !== ${JSON.stringify(Loader.instance.options)}`);
	            }
	            return Loader.instance;
	        }
	        Loader.instance = this;
	    }
	    get options() {
	        return {
	            version: this.version,
	            apiKey: this.apiKey,
	            channel: this.channel,
	            client: this.client,
	            id: this.id,
	            libraries: this.libraries,
	            language: this.language,
	            region: this.region,
	            mapIds: this.mapIds,
	            nonce: this.nonce,
	            url: this.url,
	        };
	    }
	    /**
	     * CreateUrl returns the Google Maps JavaScript API script url given the [[LoaderOptions]].
	     *
	     * @ignore
	     */
	    createUrl() {
	        let url = this.url;
	        url += `?callback=${this.CALLBACK}`;
	        if (this.apiKey) {
	            url += `&key=${this.apiKey}`;
	        }
	        if (this.channel) {
	            url += `&channel=${this.channel}`;
	        }
	        if (this.client) {
	            url += `&client=${this.client}`;
	        }
	        if (this.libraries.length > 0) {
	            url += `&libraries=${this.libraries.join(",")}`;
	        }
	        if (this.language) {
	            url += `&language=${this.language}`;
	        }
	        if (this.region) {
	            url += `&region=${this.region}`;
	        }
	        if (this.version) {
	            url += `&v=${this.version}`;
	        }
	        if (this.mapIds) {
	            url += `&map_ids=${this.mapIds.join(",")}`;
	        }
	        return url;
	    }
	    /**
	     * Load the Google Maps JavaScript API script and return a Promise.
	     */
	    load() {
	        return this.loadPromise();
	    }
	    /**
	     * Load the Google Maps JavaScript API script and return a Promise.
	     *
	     * @ignore
	     */
	    loadPromise() {
	        return new Promise((resolve, reject) => {
	            this.loadCallback((err) => {
	                if (!err) {
	                    resolve();
	                }
	                else {
	                    reject(err);
	                }
	            });
	        });
	    }
	    /**
	     * Load the Google Maps JavaScript API script with a callback.
	     */
	    loadCallback(fn) {
	        this.callbacks.push(fn);
	        this.execute();
	    }
	    /**
	     * Set the script on document.
	     */
	    setScript() {
	        if (document.getElementById(this.id)) {
	            // TODO wrap onerror callback for cases where the script was loaded elsewhere
	            this.callback();
	            return;
	        }
	        const url = this.createUrl();
	        const script = document.createElement("script");
	        script.id = this.id;
	        script.type = "text/javascript";
	        script.src = url;
	        script.onerror = this.loadErrorCallback.bind(this);
	        script.defer = true;
	        script.async = true;
	        if (this.nonce) {
	            script.nonce = this.nonce;
	        }
	        document.head.appendChild(script);
	    }
	    deleteScript() {
	        const script = document.getElementById(this.id);
	        if (script) {
	            script.remove();
	        }
	    }
	    resetIfRetryingFailed() {
	        const possibleAttempts = this.retries + 1;
	        if (this.done && !this.loading && this.errors.length >= possibleAttempts) {
	            this.deleteScript();
	            this.done = false;
	            this.loading = false;
	            this.errors = [];
	        }
	    }
	    loadErrorCallback(e) {
	        this.errors.push(e);
	        if (this.errors.length <= this.retries) {
	            const delay = this.errors.length * Math.pow(2, this.errors.length);
	            console.log(`Failed to load Google Maps script, retrying in ${delay} ms.`);
	            setTimeout(() => {
	                this.deleteScript();
	                this.setScript();
	            }, delay);
	        }
	        else {
	            this.onerrorEvent = e;
	            this.callback();
	        }
	    }
	    setCallback() {
	        window.__googleMapsCallback = this.callback.bind(this);
	    }
	    callback() {
	        this.done = true;
	        this.loading = false;
	        this.callbacks.forEach((cb) => {
	            cb(this.onerrorEvent);
	        });
	        this.callbacks = [];
	    }
	    execute() {
	        if (window.google && window.google.maps && window.google.maps.version) {
	            console.warn("Aborted attempt to load Google Maps JS with @googlemaps/js-api-loader." +
	                "This may result in undesirable behavior as script parameters may not match.");
	            this.callback();
	        }
	        this.resetIfRetryingFailed();
	        if (this.done) {
	            this.callback();
	        }
	        else {
	            if (this.loading) ;
	            else {
	                this.loading = true;
	                this.setCallback();
	                this.setScript();
	            }
	        }
	    }
	}

	/**
	 * Helper tool for dynamically loading the Google Maps Api `script` tag
	 */

	function googleMapsApiLoader(options) {
	  const loader = new Loader(options);
	  loader.load();
	}

	exports.GoogleLayer = GoogleLayer;
	exports.googleMapApiLoader = googleMapsApiLoader;

	Object.defineProperty(exports, '__esModule', { value: true });

})));
